#include <QString>
#include <QTreeView>
#include <QPushButton>
#include <QLabel>
#include <QCheckBox>
#include <QAbstractItemModel>
#include <QStandardItemModel>
#include <QSortFilterProxyModel>
#include <QModelIndex>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QMainWindow>
#include <QFileDialog>
#include <QMessageBox>

#include <iostream>
using namespace std;

#include "agepage.h"
#include "sim.h"
#include "simproxymodel.h"

AgePage::AgePage (QMainWindow *window) {
  listManagementButton = new QPushButton ("&Manage Sim Lists...");
  listManagementButton->setMaximumSize (listManagementButton->sizeHint ());
  addNewButton = new QPushButton ("Add &New Sim");
  addNewButton->setMaximumSize (addNewButton->sizeHint ());
  editButton = new QPushButton ("Edit Selected Sim");
  editButton->setMaximumSize (editButton->sizeHint ());
  editButton->setEnabled (false);
  filterButton = new QPushButton ("Filter List...");
  filterButton->setMaximumSize (filterButton->sizeHint ());
  settingsButton = new QPushButton ("&Settings...");
  settingsButton->setMaximumSize (settingsButton->sizeHint ());

  connect (listManagementButton, SIGNAL (clicked ()), this, 
           SIGNAL (newListManagementDialog ()));
  connect (addNewButton, SIGNAL (clicked ()), this,
           SIGNAL (newAddSimDialog ()));
  connect (editButton, SIGNAL (clicked ()), this, 
           SLOT (launchEditSimDialog ()));
  connect (filterButton, SIGNAL (clicked ()), this,
           SIGNAL (newFilterDialog ()));
  connect (settingsButton, SIGNAL (clicked ()), this, 
           SIGNAL (newSettingsDialog ()));

  topLayout = new QHBoxLayout ();
  topLayout->addWidget (listManagementButton);
  topLayout->setAlignment (listManagementButton, Qt::AlignLeft);
  topLayout->addWidget (addNewButton);
  topLayout->setAlignment (addNewButton, Qt::AlignRight);
  topLayout->addWidget (editButton);
  topLayout->addWidget (filterButton);
  topLayout->setAlignment (filterButton, Qt::AlignLeft);
  topLayout->addWidget (settingsButton);
  topLayout->setAlignment (settingsButton, Qt::AlignRight);

  proxyModel = new SimProxyModel (this);
  proxyModel->setDynamicSortFilter (true);

  simView = new QTreeView;
  simView->setRootIsDecorated (false);
  simView->setAlternatingRowColors (true);
  simView->setModel (proxyModel);
  simView->setSortingEnabled (true);
  simView->setSelectionBehavior (QAbstractItemView::SelectRows);

  addImageButton = new QPushButton ("Add Image");
  addImageButton->setMaximumSize (addImageButton->sizeHint ());
  addImageButton->setEnabled (false);
  imageDisplay = new QLabel ();  

  notesDisplay = new QLabel ("Notes for ");
  notesDisplay->setWordWrap (true);
  addNotesButton = new QPushButton ("Add Notes");
  addNotesButton->setMaximumSize (addNotesButton->sizeHint ());
  addNotesButton->setEnabled (false);

  mainLayout = new QVBoxLayout ();
  mainLayout->addLayout (topLayout);
  mainLayout->addWidget (simView);
  mainLayout->addWidget (addImageButton);
  mainLayout->addWidget (imageDisplay);
  mainLayout->addWidget (notesDisplay);
  mainLayout->addWidget (addNotesButton);

  setLayout (mainLayout);

  connect (simView->selectionModel (), 
             SIGNAL (currentChanged (const QModelIndex&, const QModelIndex&)), 
           this, SLOT (sendUpdate ()));

  connect (addNotesButton, SIGNAL (clicked ()), this, 
           SLOT (launchAddNotesDialog ()));
  connect (addImageButton, SIGNAL (clicked ()), this, SLOT (getImageFile ()));
}

void AgePage::setSourceModel (QAbstractItemModel *model)  {
  proxyModel->setSourceModel (model);

  for (int x = 0; x < 4; x++)
    simView->resizeColumnToContents (x);

  simView->show ();
}

QAbstractItemModel *AgePage::sourceModel ()  {
  return proxyModel->sourceModel ();
}

void AgePage::setNotesDisplay (QString n)  {
  addNotesButton->setEnabled (true);

  QModelIndex current = simView->selectionModel ()->currentIndex ();
  int proxyRow = current.row ();

  notesDisplay->setText ("Notes for " +
    proxyModel->index (proxyRow, 0).data ().toString () + ":\n\n" + n);
}

void AgePage::setImageFile (QString i)  {
  addImageButton->setEnabled (true);

  QPixmap pixmap (i);
  pixmap.load (i);

  if (!pixmap.isNull ())
    imageDisplay->setPixmap (pixmap.scaled (150, 150, Qt::KeepAspectRatio));

  else imageDisplay->setText (i);
}

void AgePage::sort ()  {
  proxyModel->sort (0, Qt::AscendingOrder);
}

void AgePage::reset ()  {
  addNotesButton->setEnabled (false);
  addImageButton->setEnabled (false);
  editButton->setEnabled (false);
  notesDisplay->setText ("Notes for ");
  imageDisplay->clear ();
  simView->selectionModel ()->reset ();
}

void AgePage::sendUpdate ()  {
  // Sends the index of the sim currently selected in the model, so that other
  // pages associated with a single sim can alter themselves.

  addNotesButton->setEnabled (true);
  addImageButton->setEnabled (true);
  editButton->setEnabled (true);

  QModelIndex current = simView->selectionModel ()->currentIndex ();
  QModelIndex source = proxyModel->mapToSource (current);

  emit update (source.row ());
}

void AgePage::launchAddNotesDialog ()  {
  QModelIndex current = simView->selectionModel ()->currentIndex ();
  QModelIndex source = proxyModel->mapToSource (current);

  emit newAddNotesDialog (source.row ());
}

void AgePage::launchEditSimDialog ()  {
  QModelIndex current = simView->selectionModel ()->currentIndex ();
  QModelIndex source = proxyModel->mapToSource (current);

  emit newEditSimDialog (source.row ());
}

void AgePage::getImageFile ()  {
  QString filename = "";
  QFileDialog fileDialog (this);
  fileDialog.setFileMode (QFileDialog::ExistingFile);
  fileDialog.setFilter ("Images (*.jpg *.gif)");
  fileDialog.setViewMode (QFileDialog::List);

  if (!fileDialog.exec ()) return;

  filename = fileDialog.selectedFiles ()[0];

  filename.replace ("\\", "/");
  QModelIndex current = simView->selectionModel ()->currentIndex ();
  QModelIndex source = proxyModel->mapToSource (current);

  emit assignImageFile (source.row (), filename);
  emit update (source.row ());
}
