#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QString>
#include <QLabel>
#include <QComboBox>
#include <QGroupBox>
#include <QScrollArea>
#include <QMessageBox>

#include "aspirationpage.h"
#include "sim.h"
#include "aspirationdisplay.h"
#include "const.h"

AspirationPage::AspirationPage (int flags)
{
  sim = NULL;
  epFlags = flags;

  nameLabel = new QLabel ("Please select a name from the List of Sims tab.");
  nameLabel->setMaximumSize (nameLabel->sizeHint ());
  nameLabel->setAlignment (Qt::AlignCenter);

  nameLayout = new QHBoxLayout;
  nameLayout->addStretch (1);
  nameLayout->addWidget (nameLabel);
  nameLayout->addStretch (1);

  hobbyLabel = new QLabel ("One True Hobby:");
  hobbyLabel->setMaximumSize (hobbyLabel->sizeHint ());
  hobbyBox = new QComboBox;
  hobbyBox->addItem ("Unknown");
  hobbyBox->addItem ("Arts & Crafts");
  hobbyBox->addItem ("Cuisine");
  hobbyBox->addItem ("Film & Literature");
  hobbyBox->addItem ("Fitness");
  hobbyBox->addItem ("Gaming");
  hobbyBox->addItem ("Music & Dance");
  hobbyBox->addItem ("Nature");
  hobbyBox->addItem ("Science");
  hobbyBox->addItem ("Sports");
  hobbyBox->addItem ("Tinkering");
  hobbyBox->setMaximumSize (hobbyBox->sizeHint ());

  hobbyLayout = new QHBoxLayout;
  hobbyLayout->addStretch (1);
  hobbyLayout->addWidget (hobbyLabel, Qt::AlignRight | Qt::AlignVCenter);
  hobbyLayout->addWidget (hobbyBox, Qt::AlignLeft | Qt::AlignVCenter);
  hobbyLayout->addStretch (1);

  displayWidget = new QWidget;
  displayLayout = new QVBoxLayout;
  displayWidget->setLayout (displayLayout);

  displayArea = new QScrollArea;
  displayArea->setWidget (displayWidget);

  mainLayout = new QVBoxLayout;
  mainLayout->addLayout (nameLayout, Qt::AlignHCenter | Qt::AlignTop);

  if (epFlags & FREETIME)
    mainLayout->addLayout (hobbyLayout, Qt::AlignHCenter | Qt::AlignTop);

  mainLayout->addWidget (displayArea);

  setLayout (mainLayout);
}

void AspirationPage::setSim (QPointer<Sim> s)  {
  sim = s;
  updateDisplay ();
}

void AspirationPage::setEPFlags (int flags)  {
  epFlags = flags;
  updateDisplay ();
}

void AspirationPage::updateDisplay ()  {
  while (mainLayout->takeAt (0));

  mainLayout->addLayout (nameLayout, Qt::AlignHCenter | Qt::AlignTop);
  if (epFlags & FREETIME) mainLayout->addLayout (hobbyLayout);
  mainLayout->addWidget (displayArea);

  while (displayLayout->takeAt (0));

  for (int x = 0; x < displayList.size (); x++)
    delete displayList[x];

  displayList.clear ();

  if (sim)  {
    nameLabel->setText ("The Aspirations of " + sim->getFullName ());
    nameLabel->setMaximumSize (nameLabel->sizeHint ());

    int currentAsp = 0;

    while (sim->getAspirationCode (currentAsp) == NONE) currentAsp++;

    while (sim->getAspirationCode (currentAsp) != NONE  &&
           sim->getAspirationCode (currentAsp) != UNAVAILABLE)  {
      AspirationDisplay *display = new AspirationDisplay (epFlags);
      display->setAspiration (sim->getAspirationCode (currentAsp),
                              sim->getSecondaryAspirationCode (currentAsp));

      int currentLTW = 0;

      while (sim->getLTW (currentAsp, currentLTW) != "")  {
        display->addLTW (sim->getLTW (currentAsp, currentLTW),
                         sim->getLTWSteps (currentAsp, currentLTW),
                         sim->getLTWCompletedSteps (currentAsp, currentLTW));

        currentLTW++;
      };

      connect (display, 
               SIGNAL (updateAspiration (AspirationDisplay*, int, int)),
               this, SLOT (updateAspiration (AspirationDisplay*, int, int)));
      connect (display, 
               SIGNAL (updateLTW (AspirationDisplay*, int, QString, int, int)),
               this, 
               SLOT (updateLTW (AspirationDisplay*, int, QString, int, int)));
      connect (display, SIGNAL (deleteMe (AspirationDisplay*)), this,
               SLOT (deleteAspirationDisplay (AspirationDisplay*)));

      displayList.append (display);
      currentAsp++;
    };

    for (int x = displayList.size () - 1; x >= 0; x--)  {
      displayLayout->addWidget (displayList[x]);
      displayList[x]->show ();
    }
  }

  else
    nameLabel->setText ("Please select a name from the List of Sims tab.");

  displayLayout->setSizeConstraint (QLayout::SetFixedSize);
}

void AspirationPage::updateAspiration (AspirationDisplay *d, int p, int s)  {
  int displayNum = -1;

  for (int x = 0; x < displayLayout->count (); x++)  {
    if (qobject_cast<AspirationDisplay*>(displayLayout->itemAt (x)->widget ())
         == d)
      displayNum = x;
  }

  if (displayNum < 0) return;

  sim->updateAspiration (p, s, displayNum);
}

void AspirationPage::updateLTW (AspirationDisplay *d, int ltwNum, QString desc,
                                int completedSteps, int totalSteps)  {
  int displayNum = -1;

  for (int x = 0; x < displayLayout->count (); x++)
    if (qobject_cast<AspirationDisplay*>(displayLayout->itemAt (x)->widget ())
           == d)
      displayNum = x;

  if (displayNum < 0) return;

  sim->updateLTW (displayNum, ltwNum, desc, completedSteps, totalSteps);  
}

void AspirationPage::deleteAspirationDisplay (AspirationDisplay *display)  {
  int displayNum = -1;

  for (int x = 0; x < displayLayout->count (); x++)
    if (qobject_cast<AspirationDisplay*>(displayLayout->itemAt (x)->widget ())
           == display)
      displayNum = x;

  if (displayNum < 0) return;

  mainLayout->removeWidget (display);
  delete display;
  resize (minimumSize ());

  sim->deleteAspiration (displayNum);
}
