#include <QPushButton>
#include <QLabel>
#include <QCheckBox>
#include <QTreeView>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QMessageBox>
#include <QAbstractItemModel>
#include <QStandardItemModel>
#include <QSortFilterProxyModel>

#include "const.h"
#include "sim.h"
#include "familypage.h"
#include "familybrowser.h"
#include "familyproxymodel.h"

FamilyPage::FamilyPage (QList< QPointer<Sim> > list)  {
  sim = NULL;
  simList = list;

  familyBrowser = NULL;

  addEditFamilyButton = new QPushButton ("Add or Edit Family");
  addEditFamilyButton->setMaximumSize (addEditFamilyButton->sizeHint ());
  kintermsButton = new QPushButton ("The Meanings of English Kinterms");
  kintermsButton->setMaximumSize (kintermsButton->sizeHint ());
  livingSimsBox = new QCheckBox ("Show only living sims");
  livingSimsBox->setChecked (true);
  livingSimsBox->setMaximumSize (livingSimsBox->sizeHint ());
  playableSimsBox = new QCheckBox ("Show only playable sims");
  playableSimsBox->setChecked (true);
  buttonLayout = new QHBoxLayout;
  buttonLayout->addWidget (addEditFamilyButton);
  buttonLayout->addWidget (kintermsButton);
  buttonLayout->addWidget (livingSimsBox);
  buttonLayout->addWidget (playableSimsBox);

  nameLabel = new QLabel ("The Family of :");

  proxyModel = new FamilyProxyModel (this);
  proxyModel->setDynamicSortFilter (true);

  familyView = new QTreeView;
  familyView->setRootIsDecorated (false);
  familyView->setAlternatingRowColors (true);
  familyView->setModel (proxyModel);
  familyView->setSortingEnabled (true);
  familyView->setSelectionBehavior (QAbstractItemView::SelectRows);

  mainLayout = new QVBoxLayout;
  mainLayout->addLayout (buttonLayout);
  mainLayout->addWidget (nameLabel);
  mainLayout->addWidget (familyView);

  setLayout (mainLayout);

  connect (addEditFamilyButton, SIGNAL (clicked ()), this,
           SLOT (createFamilyBrowser ()));
  connect (livingSimsBox, SIGNAL (toggled (bool)), this,
           SLOT (showFamily ()));
  connect (playableSimsBox, SIGNAL (toggled (bool)), this,
           SLOT (showFamily ()));
}

void FamilyPage::setSim (QPointer<Sim> s)  {
  sim = s;
 
  if (familyBrowser) familyBrowser->setSim (sim);

  showFamily ();
}

void FamilyPage::setSimList (QList< QPointer<Sim> > list)  {
  simList = list;

  if (familyBrowser) familyBrowser->setSimList (simList);
}

void FamilyPage::setSourceModel (QAbstractItemModel *model)  {
  proxyModel->setSourceModel (model);
  proxyModel->sort (0);

  familyView->resizeColumnToContents (0);
  familyView->resizeColumnToContents (1);
}

void FamilyPage::createFamilyBrowser ()  {
  if (!sim)  {
    QMessageBox::information 
      (this, "", "Please select a sim from the list first.");
    return;
  }

  if (!familyBrowser)  {
    familyBrowser = new FamilyBrowser (sim, simList);
    connect (familyBrowser, SIGNAL (closeAndUpdate ()), this, 
             SLOT (showFamily ()));
  }

  familyBrowser->setSim (sim);
  familyBrowser->show ();
}

void FamilyPage::showFamily ()  {
  displayedSimsList.clear ();

  familyView->selectionModel ()->reset ();
  proxyModel->sourceModel ()->removeRows (0, 
                               proxyModel->sourceModel ()->rowCount ());

  if (sim.isNull ())  {
    nameLabel->setText ("The Family of ");
    return;
  }

  nameLabel->setText ("The Family of " + sim->getFullName () + ":");

  QPointer<Sim> simPointer;
  int currentChild = 0;

  // Spouse and In-Laws
  simPointer = sim->getSpouse ();
  if (simPointer)  {
    if (simPointer->getGender () == MALE)
      addSim (simPointer, "Husband");
    else addSim (simPointer, "Wife");

    if (simPointer->getParent1 ())  {
      if (simPointer->getParent1 ()->getGender () == MALE)
        addSim (simPointer->getParent1 (), "Father-in-Law");
      else addSim (simPointer->getParent1 (), "Mother-in-Law");

      currentChild = 0;
      QPointer<Sim> childPtr = simPointer->getParent1 ()->getChild (0);
      while (childPtr)  {
        if (childPtr == simPointer)  {
          childPtr = simPointer->getParent1 ()->getChild (++currentChild);
          continue;
        }

        if (childPtr->getGender () == MALE)
          addSim (childPtr, "Brother-in-Law");
        else addSim (childPtr, "Sister-in-Law");

        childPtr = simPointer->getParent1 ()->getChild (++currentChild);
      }
    }

    if (simPointer->getParent2 ())  {
      if (simPointer->getParent2 ()->getGender () == MALE)
        addSim (simPointer->getParent2 (), "Father-in-Law");
      else addSim (simPointer->getParent2 (), "Mother-in-Law");

      currentChild = 0;
      QPointer<Sim> childPtr = simPointer->getParent2 ()->getChild (0);
      while (childPtr)  {
        if (childPtr == simPointer)  {
          childPtr = simPointer->getParent2 ()->getChild (++currentChild);
          continue;
        }

        if (childPtr->getGender () == MALE)
          addSim (childPtr, "Brother-in-Law");
        else addSim (childPtr, "Sister-in-Law");

        childPtr = simPointer->getParent2 ()->getChild (++currentChild);
      }
    }

    currentChild = 0;
    while (simPointer->getChild (currentChild))  {
      if (simPointer->getChild (currentChild)->isChildOf (sim))  {
        currentChild++;
        continue;
      }

      if (simPointer->getChild (currentChild)->getGender () == MALE)
        addSim (simPointer->getChild (currentChild), "Step-Son");
      else addSim (simPointer->getChild (currentChild), "Step-Daughter");

      currentChild++;
    }
  }

  // Children -> grandchildren -> great-grandchildren...
  currentChild = 0;
  while (sim->getChild (currentChild))  {
    simPointer = sim->getChild (currentChild);

    if (simPointer->getGender () == MALE)
      addSim (simPointer, "Son");
    else addSim (simPointer, "Daughter");

    if (simPointer->getSpouse ())  {
      if (simPointer->getSpouse ()->getGender () == MALE)
        addSim (simPointer->getSpouse (), "Son-in-Law");
      else addSim (simPointer->getSpouse (), "Daughter-in-Law");
    }

    int currentGrandchild = 0;
    while (simPointer->getChild (currentGrandchild))  {
      addRecursively (simPointer->getChild (currentGrandchild), "Grandson",
                      "Granddaughter", "Great-");
      currentGrandchild++;
    };

    currentChild++;
  };

  // Father -> brother -> nephew -> grandnephew -> great-grandnephew...
  if (sim->getParent1 ())  {
    simPointer = sim->getParent1 ();

    if (simPointer->getGender () == MALE)
      addSim (simPointer, "Father");
    else addSim (simPointer, "Mother");

    if (simPointer->getSpouse () && 
        !(sim->isChildOf (simPointer->getSpouse ())))  {
      if (simPointer->getSpouse ()->getGender () == MALE)
        addSim (simPointer->getSpouse (), "Step-Father");
      else addSim (simPointer->getSpouse (), "Step-Mother");
    }

    currentChild = 0;
    QPointer<Sim> childPtr = simPointer->getChild (currentChild);

    while (childPtr)  {
      if (childPtr == sim)  {
        childPtr = simPointer->getChild (++currentChild);
        continue;
      }

      if (childPtr->getParent2 () && 
          childPtr->getParent2 () == sim->getParent2 ())  {
        if (childPtr->getGender () == MALE)
          addSim (childPtr, "Brother");
        else addSim (childPtr, "Sister");
      }

      else  {
        if (childPtr->getGender () == MALE)
          addSim (childPtr, "Half-Brother");
        else addSim (childPtr, "Half-Sister");
      }

      if (childPtr->getSpouse ())  {
        if (childPtr->getSpouse ()->getGender () == MALE)
          addSim (childPtr->getSpouse (), "Brother-in-Law");
        else addSim (childPtr->getSpouse (), "Sister-in-Law");
      }

      int currentGrandchild = 0;
      QPointer<Sim> grandchildPtr = childPtr->getChild (currentGrandchild);

      while (grandchildPtr)  {
        if (grandchildPtr->getGender () == MALE)
          addSim (grandchildPtr, "Nephew");
        else addSim (grandchildPtr, "Niece");

        int currentNextChild = 0;
        QPointer<Sim> nextChildPtr = grandchildPtr->getChild (currentNextChild);

        while (nextChildPtr)  {
          addRecursively (nextChildPtr, "Grandnephew", "Grandniece", "Great-");
          nextChildPtr = grandchildPtr->getChild (++currentNextChild);
        }

        grandchildPtr = childPtr->getChild (++currentGrandchild);
      }

      childPtr = simPointer->getChild (++currentChild);
    }
  }

  if (sim->getParent2 ())  {
    simPointer = sim->getParent2 ();

    if (simPointer->getGender () == MALE)
      addSim (simPointer, "Father");
    else addSim (simPointer, "Mother");

    if (simPointer->getSpouse () && 
        !(sim->isChildOf (simPointer->getSpouse ())))  {
      if (simPointer->getSpouse ()->getGender () == MALE)
        addSim (simPointer->getSpouse (), "Step-Father");
      else addSim (simPointer->getSpouse (), "Step-Mother");
    }

    currentChild = 0;
    QPointer<Sim> childPtr = simPointer->getChild (currentChild);

    while (childPtr)  {
      if (childPtr == sim)  {
        childPtr = simPointer->getChild (++currentChild);
        continue;
      }

      if (childPtr->getParent1 () && 
          childPtr->getParent1 () == sim->getParent1 ())  {
        if (childPtr->getGender () == MALE)
          addSim (childPtr, "Brother");
        else addSim (childPtr, "Sister");
      }

      else  {
        if (childPtr->getGender () == MALE)
          addSim (childPtr, "Half-Brother");
        else addSim (childPtr, "Half-Sister");
      }

      if (childPtr->getSpouse ())  {
        if (childPtr->getSpouse ()->getGender () == MALE)
          addSim (childPtr->getSpouse (), "Brother-in-Law");
        else addSim (childPtr->getSpouse (), "Sister-in-Law");
      }

      int currentGrandchild = 0;
      QPointer<Sim> grandchildPtr = childPtr->getChild (currentGrandchild);

      while (grandchildPtr)  {
        if (grandchildPtr->getGender () == MALE)
          addSim (grandchildPtr, "Nephew");
        else addSim (grandchildPtr, "Niece");

        int currentNextChild = 0;
        QPointer<Sim> nextChildPtr = grandchildPtr->getChild (currentNextChild);

        while (nextChildPtr)  {
          addRecursively (nextChildPtr, "Grandnephew", "Grandniece", "Great-");
          nextChildPtr = grandchildPtr->getChild (++currentNextChild);
        }

        grandchildPtr = childPtr->getChild (++currentGrandchild);
      }

      childPtr = simPointer->getChild (++currentChild);
    }
  }

  // Grandfather -> uncle -> 1st cousin -> 1st cousin once removed...
  for (int x = 0; x < 4; x++)  {
    if (x == 0)  {
      if (sim->getParent1 ())
        simPointer = sim->getParent1 ()->getParent1 ();
      else continue;
    }

    if (x == 1)  {
      if (sim->getParent1 ())
        simPointer = sim->getParent1 ()->getParent2 ();
      else continue;
    }

    if (x == 2)  {
      if (sim->getParent2 ())
        simPointer = sim->getParent2 ()->getParent1 ();
      else continue;
    }

    if (x == 3)  {
      if (sim->getParent2 ())
        simPointer = sim->getParent2 ()->getParent2 ();
      else continue;
    }

    if (simPointer.isNull ()) continue;

    if (simPointer->getGender () == MALE)
      addSim (simPointer, "Grandfather");
    else addSim (simPointer, "Grandmother");

    currentChild = 0;
    QPointer<Sim> childPtr = simPointer->getChild (currentChild);

    while (childPtr)  {
      if (sim->isChildOf (childPtr))  {
        childPtr = simPointer->getChild (++currentChild);
        continue;
      }

      if (childPtr->getGender () == MALE)
        addSim (childPtr, "Uncle");
      else addSim (childPtr, "Aunt");

      if (childPtr->getSpouse ())  {
        if (childPtr->getSpouse ()->getGender () == MALE)
          addSim (childPtr->getSpouse (), "Uncle");
        else addSim (childPtr->getSpouse (), "Aunt");
      }

      int currentGrandchild = 0;
      QPointer<Sim> grandchildPtr = childPtr->getChild (currentGrandchild);

      while (grandchildPtr)  {
        addCousinsRecursively (grandchildPtr, "First", 0);
        grandchildPtr = childPtr->getChild (++currentGrandchild);
      }
      
      childPtr = simPointer->getChild (++currentChild);
    }

    QPointer<Sim> parentPointer = simPointer->getParent1 ();
    if (parentPointer)
      addSeriesRecursively (parentPointer, "Grandfather", "Uncle",
                            "Grandmother", "Aunt", 3, simPointer);

    parentPointer = simPointer->getParent2 ();
    if (parentPointer)
      addSeriesRecursively (parentPointer, "Grandfather", "Uncle",
                            "Grandmother", "Aunt", 3, simPointer);
  }

  // Great-#1 -> great-#2 -> 1st cousin (x-2) times removed ->
  // 2nd cousin (x-3) times removed -> etc... -> (x-1)th cousin ->
  // (x-1)th cousin once removed...
  // (Beginning with x = 3)

  proxyModel->sort (1);
}

void FamilyPage::addSim (QPointer<Sim> s, QString relationship)  {
  QAbstractItemModel *source = proxyModel->sourceModel ();

  if ((!(livingSimsBox->isChecked ()) || s->isAlive ()) && 
      (!(playableSimsBox->isChecked ()) || s->getHousehold () != "At Large") &&
       s != sim)  {
    if (displayedSimsList.contains (s))  {
      int number = 0;

      for (int x = 0; x < displayedSimsList.size (); x++)
        if (displayedSimsList[x] == s)
          number = x;

      QString original = source->index (number, 1).data ().toString ();
      if (!original.contains (relationship))
        source->setData (source->index (number, 1), original + "; " + 
                         relationship, Qt::DisplayRole);
    }

    else  {
      displayedSimsList.prepend (s);

      source->insertRow (0);
      source->setData (source->index (0, 0), s->getFullName(), Qt::DisplayRole);
      source->setData (source->index (0, 1), relationship, Qt::DisplayRole);
    }
  }

   proxyModel->sort (0);

  familyView->resizeColumnToContents (0);
  familyView->resizeColumnToContents (1);
}

void FamilyPage::addRecursively (QPointer<Sim> ancestor, QString male, 
                                 QString female, QString prefix)  {
  if (ancestor->getGender () == MALE)
    addSim (ancestor, male);
  else addSim (ancestor, female);

  int currentChild = 0;
  while (ancestor->getChild (currentChild))  {
    addRecursively (ancestor->getChild (currentChild), prefix + male,
        prefix + female, prefix);
    currentChild++;
  };
}

void FamilyPage::addCousinsRecursively (QPointer<Sim> cousin, QString enth,
                                        int times)  {
  QString relation = enth + " Cousin";

  if (times > 0)
    relation += ", " + ntimes (times) + " Removed";

  addSim (cousin, relation);

  int currentChild = 0;
  while (cousin->getChild (currentChild))  {
    addCousinsRecursively (cousin->getChild (currentChild), enth, times + 1);
    currentChild++;
  }
}

void FamilyPage::addSeriesRecursively (QPointer<Sim> ancestor, QString firstM,
                                       QString secondM, QString firstF,
                                       QString secondF, int seriesNum,
                                       QPointer<Sim> exclude)  {
  // Great-#1 -> great-#2 -> 1st cousin (x-2) times removed ->
  // 2nd cousin (x-3) times removed -> etc... -> (x-1)th cousin ->
  // (x-1)th cousin once removed...
  // (Beginning with x = 3)
  //
  // Precondition: ancestor is not null

  if (ancestor->getGender () == MALE)
    addSim (ancestor, "Great-" + firstM);
  else addSim (ancestor, "Great-" + firstF);

  int currentChild = 0;
  QPointer<Sim> childPtr = ancestor->getChild (currentChild);

  while (childPtr)  {
    if (childPtr == exclude)  {
      childPtr = ancestor->getChild (++currentChild);
      continue;
    }

    if (childPtr->getGender () == MALE)
      addSim (childPtr, "Great-" + secondM);
    else addSim (childPtr, "Great-" + secondF);

    int currentGrandchild = 0;
    QPointer<Sim> grandchildPtr = childPtr->getChild (currentGrandchild);

    while (grandchildPtr)  {
      countDownCousinsRecursively (grandchildPtr, 1, seriesNum);
      grandchildPtr = childPtr->getChild (++currentGrandchild);
    }

    childPtr = ancestor->getChild (++currentChild);
  }

  QPointer<Sim> parentPointer = ancestor->getParent1 ();
  if (parentPointer)
    addSeriesRecursively (parentPointer, "Great-" + firstM, "Great-" + secondM,
                          "Great-" + firstF, "Great-" + secondF, seriesNum + 1,
                          ancestor);

  parentPointer = ancestor->getParent2 ();
  if (parentPointer)
    addSeriesRecursively (parentPointer, "Great-" + firstM, "Great-" + secondM,
                          "Great-" + firstF, "Great-" + secondF, seriesNum + 1,
                          ancestor);
}

void FamilyPage::countDownCousinsRecursively (QPointer<Sim> cousin, 
                                              int cousinNum, int seriesNum)  {
  QString relation = "";

  if (seriesNum > 2)
    relation += nth (cousinNum) + " Cousin, " + ntimes (seriesNum - 2) + 
                " Removed";

  addSim (cousin, relation);

  int currentChild = 0;
  QPointer<Sim> childPtr = cousin->getChild (currentChild);

  if (seriesNum == 3)
    while (childPtr)  {
      addCousinsRecursively (childPtr, nth (cousinNum + 1), 0);
      childPtr = cousin->getChild (++currentChild);
    }

  else
    while (childPtr)  {
      countDownCousinsRecursively (childPtr, cousinNum + 1, seriesNum - 1);
      childPtr = cousin->getChild (++currentChild);
    }
}

QString FamilyPage::ntimes (int num)  {
  if (num == 1) return "Once";
  if (num == 2) return "Twice";

  QString relation = "";

  switch (num)  {
    case 3: relation += "Three "; break;
    case 4: relation += "Four "; break;
    case 5: relation += "Five "; break;
    case 6: relation += "Six "; break;
    case 7: relation += "Seven "; break;
    case 8: relation += "Eight "; break;
    case 9: relation += "Nine "; break;
    case 10: relation += "Ten "; break;
    case 11: relation += "Eleven "; break;
    case 12: relation += "Twelve "; break;
    case 13: relation += "Thirteen "; break;
    case 14: relation += "Fourteen "; break;
    case 15: relation += "Fifteen "; break;
    case 16: relation += "Sixteen "; break;
    case 17: relation += "Seventeen "; break;
    case 18: relation += "Eighteen "; break;
    case 19: relation += "Nineteen "; break;
    case 20: relation += "Twenty "; break;
    default: relation += "Many "; break;
  }

  relation += "Times";
  return relation;
}

QString FamilyPage::nth (int num)  {
    switch (num)  {
    case 1: return "First"; break;
    case 2: return "Second"; break;
    case 3: return "Third"; break;
    case 4: return "Fourth"; break;
    case 5: return "Fifth"; break;
    case 6: return "Sixth"; break;
    case 7: return "Seventh"; break;
    case 8: return "Eighth"; break;
    case 9: return "Ninth"; break;
    case 10: return "Tenth"; break;
    case 11: return "Eleventh"; break;
    case 12: return "Twefth"; break;
    case 13: return "Thirteenth"; break;
    case 14: return "Fourteenth"; break;
    case 15: return "Fifteenth"; break;
    case 16: return "Sixteenth"; break;
    case 17: return "Seventeenth"; break;
    case 18: return "Eighteenth"; break;
    case 19: return "Nineteenth"; break;
    case 20: return "Twentieth"; break;
    default: return "Distant";
  }
}
